/* tosh_lcd.c -- set LCD parameters of a Toshiba laptop.
 *
 * Copyright (c) 1998-2000  Jonathan A. Buzzard (jonathan@buzzard.org.uk)
 *                        for architecture, device driver, and libraries
*/
#define VERSION_STRING "\
Tosh_LCD version 1.0.1\n"
/* 
 * Copyright (c) 2000-2001  Christian Gennerat <christian.gennerat@polytechnique.org>
 *                        for this small program
 *

 * Wed Apr 19 12:35:27 2000 <christian.gennerat@polytechnique.org> 1.0.0
 - Initial creation    
 * mer jan 24 11:22:34 2001 <christian.gennerat@polytechnique.org> 1.0.1
 - make rpm package
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 */

static char const rcsid[]="$Id: tosh_lcd.c V1.0.1 mer jan 24 11:22:34 2001 $";


#include<stdio.h>
#include<stdlib.h>
#include<unistd.h>

#include"sci.h"
#include"hci.h"

#define USAGE_STRING "\n\
Usage: tosh_lcd [-option]\n\
Set LCD parameters of a Toshiba Libretto\n\n\
  -n,	--narrow	narrow display (console mode)\n\
  -w,	--wide		wide   display (console mode)\n\
  -f,-0,--off		switch off LCD backlight\n\
  -d,-1,--dark		dark LCD screen\n\
  -D,-2,--dim		dim  LCD screen\n\
  -s,-3,--semi		semi-bright LCD screen\n\
  -b,-4,--bright	bright LCD screen\n\
  -h,	--help		display this help message\n\
  -v,	--version	display version\n\
Report bugs to christian.gennerat@m4x.org\n"

enum {
	DISPLAY_STRETCH  = 0x0002,
	LCD_BACKLIGHT = 0x0004,
};


/*
 * check the alarm features of the laptop
 */
int LibSupport()
{
	SMMRegisters reg;
	int support;

	support = 0x0000;
	reg.ecx = 0;
	reg.edx = 0;
	SciOpenInterface();

	reg.ebx = SCI_DISPLAY_STRETCH;
	if (SciGet(&reg)==SCI_SUCCESS) {
		support |= DISPLAY_STRETCH;
	}

	reg.ebx = SCI_LCD_BACKLIGHT;
	if (SciGet(&reg)==SCI_SUCCESS) {
		support |= LCD_BACKLIGHT;
	}

	reg.edx = 0;
	SciCloseInterface();
	
	return support;
}

void BacklightOn()
{
	SMMRegisters reg;

	reg.eax = HCI_SET;
	reg.ebx = HCI_BACKLIGHT;
	reg.ecx = HCI_ENABLE;
	HciFunction(&reg);

	return;
}


void BacklightOff()
{
	SMMRegisters reg;

	reg.eax = HCI_SET;
	reg.ebx = HCI_BACKLIGHT;
	reg.ecx = HCI_DISABLE;
	HciFunction(&reg);

	return;
}


int main(int argc, char *argv[])
{
	SMMRegisters reg;
	int i,version,hibern,width,light,support;

	if (argc!=2)  {
			printf(rcsid);
			printf(USAGE_STRING);
			exit(0);
			}

	reg.ebx = 0;
	reg.ecx = 0;
	reg.edx = 0;
	/* do some quick checks on the laptop */

	if (SciSupportCheck(&version)==1) {
		fprintf(stderr, "alarm: this computer is not supported\n");
		return 1;
	}

	/* check this model supports what we want */

	support = LibSupport();
	if (support==0) {
		fprintf(stderr, 
		"Lib: feature not supported on this model.\n");
		return 1;
	}

	/* process command line arguments */

	light = -1;
	width = -1;
	hibern = -1;
	for (i=1;i<argc;i++) {
		if ((!strcmp(argv[i], "-h")) || (!strcmp(argv[i], "--help"))) {
			printf(USAGE_STRING);
			exit(0);
		}
		else if ((!strcmp(argv[i], "-v")) 
			|| (!strcmp(argv[i], "--version"))) {
			printf(VERSION_STRING);
			exit(0);
		}


		else if ((!strcmp(argv[i], "-w")) 
			|| (!strcmp(argv[i], "--wide"))) {
			width = 1;
		}		
		else if ((!strcmp(argv[i], "-n")) 
			|| (!strcmp(argv[i], "--narrow"))) {
			width = 0;
		}		


		else if ((!strcmp(argv[i], "-d")) 
			|| (!strcmp(argv[i], "-1"))
			|| (!strcmp(argv[i], "--dark"))) {
			light = SCI_BACK_DARK;
		}		
		else if ((!strcmp(argv[i], "-D")) 
			|| (!strcmp(argv[i], "-2"))
			|| (!strcmp(argv[i], "--dim"))) {
			light = SCI_BACK_DIM;
		}		
		else if ((!strcmp(argv[i], "-s")) 
			|| (!strcmp(argv[i], "-3"))
			|| (!strcmp(argv[i], "--semi"))) {
			light = SCI_BACK_SEMI;
		}		
		else if ((!strcmp(argv[i], "-b")) 
			|| (!strcmp(argv[i], "-4"))
			|| (!strcmp(argv[i], "--bright"))) {
			light = SCI_BACK_BRIGHT;
		}		
		else if ((!strcmp(argv[i], "-f")) 
			|| (!strcmp(argv[i], "-0"))
			|| (!strcmp(argv[i], "--off"))) {
			BacklightOff();
		}		
	}

	if (width!=-1) {
		SciOpenInterface();
		reg.ebx = SCI_DISPLAY_STRETCH;
		reg.ecx = width;
		reg.edx = 0;
		if (!SciSet(&reg))
			printf("Lib: Screen width updated: %d\n",width);
		SciCloseInterface();
	}

	if (light!=-1) {
		SciOpenInterface();
		reg.ebx = SCI_LCD_BACKLIGHT;
		reg.ecx = light;
		reg.edx = 0;
		if (!SciSet(&reg))
			printf("Lib: Screen light updated: %d\n",light);
		SciCloseInterface();
		BacklightOn();
	}

	return 0;
}
